/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "interpUtil.h"
#include "fitsio.h"
#include "imaging_detector.h"

/**********************************************************
 * MIDI calibration of photometric pixel coordinates.  
 * Copy distortion polynomials from the imaging_detector on
 * one the MIDI data file to another.
 * Usage:
 *    CopyCal  infile reffile
 *        infile  The name of the file whose IMAGING_DETECTOR
 *                table is to be updated.
 *        reffile MIDI file with IMAGING_DETECTOR containing the
 *                distortion polynomials.
 ************************************************************* */

int main (int argc, char** argv) {

   int ireg, ierr, ncoefX, ncoefY, retCode=1;
   char *infile, *reffile;
   fitsfile *inFile=NULL, *refFile=NULL;
   imaging_detector_table_def *inDetect=NULL, *refDetect=NULL;
   float coef[2][MAXCOEF];

   ierr = 0;

   /* parse inputs */
   infile  = argv[1];
   reffile = argv[2];

   /* get reference IMAGING_DETECTOR */
    fits_open_file(&refFile, reffile, READONLY, &ierr);
    if (ierr !=0) {
      fits_report_error(stderr, ierr);
      return retCode;
    }

    /*     Get detector section */
    refDetect = read_imaging_detector_table(refFile, &ierr);
    if (ierr !=0) {
      fprintf(stderr,"error %i reading detector table in %s\n", 
	     ierr, reffile);
      fits_report_error(stderr, ierr);
      return retCode;
    }

    /*   Close file  */
    fits_close_file(refFile, &ierr);
    if (ierr !=0) {
      fits_report_error(stderr, ierr);
      return retCode;
    }

   /* get  IMAGING_DETECTOR to update */
    fits_open_file(&inFile, infile, READWRITE, &ierr);
    if (ierr !=0) {
      fits_report_error(stderr, ierr);
      return retCode;
    }

    /*     Get detector section */
    inDetect = read_imaging_detector_table(inFile, &ierr);
    if (ierr !=0) {
      fprintf(stderr,"error %i reading detector table in %s\n", 
	      ierr, infile);
      fits_report_error(stderr, ierr);
      return retCode;
    }

     /* Check number of regions */
    if (inDetect->nregion != refDetect->nregion) {
      fprintf (stderr," Files have different number of regions %d != %d\n",
	       inDetect->nregion,refDetect->nregion);
      return retCode;
    }

    /* Check Corners, sizes */
    for (ireg=0; ireg<inDetect->nregion; ireg++) {
      if ((inDetect->corner[ireg][0] != refDetect->corner[ireg][0]) || 
	  (inDetect->corner[ireg][1] != refDetect->corner[ireg][1]) || 
	  (inDetect->naxis[ireg][0] != refDetect->naxis[ireg][0]) || 
	  (inDetect->naxis[ireg][1] != refDetect->naxis[ireg][1])) {
	fprintf (stderr," Region %d incompatable \n",ireg);
	fprintf (stderr,"  corner (%d,%d) not (%d,%d)\n or size  (%d,%d) not (%d,%d)\n",
		 inDetect->corner[ireg][0],inDetect->corner[ireg][1], 
		 refDetect->corner[ireg][0], refDetect->corner[ireg][1],
		 inDetect->naxis[ireg][0],inDetect->naxis[ireg][1], 
		 refDetect->naxis[ireg][0], refDetect->naxis[ireg][1]);
	return retCode;
      }
    } /* end loop over regions */
    
    /* update detector */
    for (ireg=0; ireg<inDetect->nregion; ireg++) {
      minrtsGetPoly  (refDetect, 0, ireg,  &ncoefX, &coef[0][0]);
      minrtsGetPoly  (refDetect, 1, ireg,  &ncoefY, &coef[1][0]);
      minrtsSetPoly  (inDetect, 0, ireg,  ncoefX, &coef[0][0]);
      minrtsSetPoly  (inDetect, 1, ireg,  ncoefY, &coef[1][0]);
    }  
    
    /*   Write output */
    write_imaging_detector_table(inFile, inDetect, &ierr);
    if (ierr !=0) {
      fprintf(stderr,"error %i writing detector table in %s\n", 
	      ierr, infile);
      fits_report_error(stderr, ierr);
      return retCode;
    }
    
    /*   Close file  */
    fits_close_file(inFile, &ierr);
    if (ierr !=0) {
      fits_report_error(stderr, ierr);
      return retCode;
    }
    
    /* cleanup */
    kill_imaging_detector_table_def (inDetect);
    kill_imaging_detector_table_def (refDetect);
    return 0;
} /* end main */

